﻿-- Priority List
-- by Fluffies- EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L
local C = addonTable.C

local addon	= PriorityList
local items = addon.ItemFunctions
local chars = addon.CharFunctions
local comm = addon.CommFunctions

-- list version: this will be checked whenever the list has to be imported/exported
-- increment this number if the list structure changes
-- and apply a list upgrade fix in Init()
addon.version = 1004
-- history
-- 1001/1002 - Items_List structure changes
-- 1003 - converted Items_List to PLWish_List (character focused structure)
-- 1004 - PriorityList new structure

local AddColor = addonTable.AddColor

addon.playername = UnitName("player")

addon.prio = { {}, {}, {} }
addon.import_list = {}

function addon:IsSuperUser()
 return PLCharOptions and PLCharOptions["superuser"]
end

function addon:GeneratePrio()
 wipe(addon.prio[1])
 wipe(addon.prio[2])
 wipe(addon.prio[3])

 for name, chardata in pairs(PLWish_List) do
  for diff, d in pairs(chardata) do
   for list, l in pairs(d.lists) do
    local counter = 1
    for k,itemID in pairs(l) do
     local c = {}
     if not addon.prio[diff][itemID] then
 	  addon.prio[diff][itemID] = {}
	 end
	 local prio_gap = (list - 1) * C.PRIO_GAP
	 local item = chars:GetLootedItem(name, diff, itemID)
	 if item then -- if it's already looted, use prio to store the looted item bonus score
	  local primary, tertiary = items:GetBonusScore(items:ExtractBonuses(item.link))
	  c.prio = C.UPGRADE_GAP + primary + prio_gap
	  c.tertiary = tertiary
	  if item.personal then counter = counter - 1 end
	 else	 
 	  c.prio = counter + prio_gap
	 end
	 c.name = name
	 table.insert(addon.prio[diff][itemID], c)
	 counter = counter + 1
    end
	for k,itemID in pairs(l) do
	 table.sort(addon.prio[diff][itemID], function(a,b) return a.prio < b.prio end)
	end
   end
  end
 end
end

function addon:Item2Token(itemID)
 for k,v in pairs(C.token_tables) do
  if tContains(v, itemID) then
   return k
  end
 end

 return itemID
end

-- loot window - item tooltip
hooksecurefunc(GameTooltip, "SetLootItem", function(tooltip, slot)
 local link = GetLootSlotLink(slot)

 if not addon:IsSuperUser() or not LootSlotHasItem(slot) or not link then return end

 link = strtrim(link)
 local itemID = items:ExtractItemID(link)

 if not itemID then return end

 local difficulty = items:GetDifficulty(link)
 local lines = {}

 if addon.prio[difficulty][itemID] then
  local last_prio, last_list, last_tertiary
  local wasUpgrade
  local i = 0
  for k,v in pairs(addon.prio[difficulty][itemID]) do
   local prio = v.prio % C.PRIO_GAP
   local isUpgrade = prio >= C.UPGRADE_GAP
   local tertiary = 0

   if isUpgrade then -- use prio to store the score differnce between the looted item and the new one
    prio = prio - C.UPGRADE_GAP
	local score1, score2 = items:GetBonusScore(items:ExtractBonuses(link))
	prio = math.floor(prio * 100) / 100
	prio = math.max(0, score1 - prio)
	tertiary = math.max(0, score2 - v.tertiary)
   end

   if prio > 0.09 or tertiary > 0 then
    local list = math.ceil(v.prio / C.PRIO_GAP)
    if not isUpgrade and list ~= last_list then
     i = i + 1
     lines[i] = AddColor(L["list"..list], "LIST_BLUE")
 	 last_prio = nil
	 wasUpgrade = nil
    elseif isUpgrade and not wasUpgrade or list ~= last_list then -- first upgrade
     i = i + 1
     lines[i] = AddColor(L["list"..list].." "..L["upgrades"], "LIST_BLUE")
	 prio = math.floor(prio * 100) / 100
   	 --last_prio = prio
    end
    if last_prio and last_prio == prio and isUpgrade == wasUpgrade and last_list == list and last_tertiary == tertiary then
     lines[i] = lines[i]..AddColor("\\", "LIST_GREEN")..AddColor(v.name, C.list_color[list])
    else
     i = i + 1
     lines[i] = isUpgrade and AddColor("+"..prio, "LIGHT_GREEN").." "..AddColor("("..tertiary..")", "ORANGE").." "..AddColor(v.name, C.list_color[list])
	                       or AddColor("#"..prio, "LIST_YELLOW").." "..AddColor(v.name, C.list_color[list])
    end
    last_list = list
    wasUpgrade = isUpgrade
   end
   last_prio = prio
   last_tertiary = tertiary
  end
 end

 for k,v in pairs(lines) do
  if k == 1 then
   --tooltip:AddLine(AddColor(L["item_goes_to"], "LIGHT_BLUE"))
   tooltip:AddLine(v)
   --tooltip:AddLine(AddColor(L["next_needers"], "LIGHT_BLUE"))
  else
   tooltip:AddLine(v)
  end
 end
 tooltip:Show()
end)

function addon:Initialize(forceShow)
 if PLMainframe then
  if forceShow == true or not PLMainframe:IsVisible() then
   PLMainframe:Show()
  else
   PLMainframe:Hide()
  end
 else
  addon:createMainframe()
 end
end

-- SetHyperLink override
local oldSetHyperLinkFunc = ItemRefTooltip.SetHyperlink
function ItemRefTooltip.SetHyperlink(tooltip, itemstring, ...)
 
 -- custom import hyperlink
 if string.sub(itemstring, 1, 12) == "prioritylist" then
  comm:SetHyperlink(tooltip, itemstring)
  return
 end

 oldSetHyperLinkFunc(tooltip, itemstring, ...)
 
 local _, link = GetItemInfo(itemstring)
 if not link or not pl_options["ttlinkbtn"] then 
  if ItemRefTooltip.PL_additem_btn then ItemRefTooltip.PL_additem_btn:Hide() end 
  return
 end
 
 local f = ItemRefTooltip.PL_additem_btn
 if not ItemRefTooltip.PL_additem_btn then
  f = CreateFrame("Button", "PLTooltipButton", tooltip, "UIPanelButtonTemplate")
  f:SetWidth(100)
  f:SetPoint("BOTTOMRIGHT", tooltip, "BOTTOMRIGHT", -10, 10);
  f:SetText(L["pickup"])
  f:SetScript("OnClick", function(self)
                           tooltip:Hide()
						   PickupItem(self.link)
						   addon:Initialize(true)
						   if PLWish_List[addon.playername] then
						    addon:EventPulse("select_char", addon.playername)
						   else
						    chars:New(addon.playername, true)
						   end
                          end)
  ItemRefTooltip.PL_additem_btn = f
 end
 f.link = link
 tooltip:AddDoubleLine("                    ","                    ")
 tooltip:AddLine(addonTable.AddColor(L["addon_name"], "LIGHT_BLUE"))
 f:Show()
 tooltip:Show()
end

-- eventhandler
addon.eventhandler = CreateFrame("Frame")
local function onEvent(self, event, ...)
 addon.eventhandler[event](...)
end
addon.eventhandler:SetScript("OnEvent", onEvent)

addon.eventhandler:RegisterEvent("CHAT_MSG_LOOT")
function addon.eventhandler.CHAT_MSG_LOOT(...)

 local s, _, _, _, name = ...
 
 if not PLWish_List[name] then return end
 
 local self_strings  = {LOOT_ITEM_SELF, LOOT_ITEM_BONUS_ROLL_SELF, LOOT_ITEM_PUSHED_SELF}
 local other_strings = {LOOT_ITEM, LOOT_ITEM_BONUS_ROLL, LOOT_ITEM_PUSHED}
 local link, index
 local tmp_s = ""

 if name == addon.playername then
  for i = 1,3 do
   local tmp = string.gsub(self_strings[i], "%%s", "(.+)")
   link = select(3, string.find(s, tmp))
   if link then
    index = i
    break
   end
  end
 else
  for i = 1,3 do
   local tmp = string.gsub(other_strings[i], "%%s", "(.+)")
   link = select(4, string.find(s, tmp))
   if link then
    index = i
    break
   end
  end
 end

 if not link then
  return
 end

 if index == 1 then
  chars:SetLooted(name, link, true)
 else -- bonus roll and other sources
  chars:SetLooted(name, link, true, true)
 end
end

function addon:ConvertPL2Lists()
 if not PLWish_List then 
  --print("No PL2 lists found")
  return
 end
 
 local tmp = PLWish_List
 PLWish_List = {}
 
 local diff = 2

 for name,data in pairs(tmp) do
  chars:New(name)
  for k,v in pairs(data) do
   PLWish_List[name][diff].lists[1][k] = math.abs(v)
   if v < 0 then
    PLWish_List[name][diff].looted[-v] = { link = items:GenerateLink(-v, nil, nil, diff), time = time() }
   end
  end
 end
end

addon.eventhandler:RegisterEvent("VARIABLES_LOADED")
function addon.eventhandler.VARIABLES_LOADED(...)
 if not PLWish_List then PLWish_List = {} end
 if not PLCharOptions then PLCharOptions = {} end

 if not pl_options then pl_options = {} end
 if pl_options["ttlinkbtn"]    		 == nil then pl_options["ttlinkbtn"] = true end
 if pl_options["autosync"]     		 == nil then pl_options["autosync"]  = false end
 if pl_options["confirm_edit"] 		 == nil then pl_options["confirm_edit"] = true end
 if pl_options["officer_mode"] 		 == nil then pl_options["officer_mode"] = false end
 if pl_options["short_looted_names"] == nil then pl_options["short_looted_names"] = true end
 if pl_options["show_all_chars"]     == nil then pl_options["show_all_chars"] = false end

 for i = 1,C.numBonuses do
  if pl_options["bonus_value_"..i] == nil then pl_options["bonus_value_"..i] = C.bIDBaseValues[i] end
 end

 if pl_options["version"] and pl_options["version"] < 1004 then
  addon:ConvertPL2Lists()
 end
 pl_options["version"] = addon.version

 -- GetGuildInfo is not available yet
 C_Timer.After(4, function()
  local guild, rank, index = GetGuildInfo("player")
  addon.guildname = guild or ""
  if not addon:IsSuperUser() and IsInGuild() then
   if guild and rank and index == 0 then
    PLCharOptions["superuser"] = guild
    print(L["SU_GM"])
   end
  end
 end)

 addon:GeneratePrio()
end

-- slash command
SLASH_PRIORITYLIST1 = "/prioritylist"
SLASH_PRIORITYLIST2 = "/pl"
SlashCmdList["PRIORITYLIST"] = addon.Initialize